// -------------------------------------------------------------
// Model Shader
// + reflection rendering
// + render to shadow map
// -------------------------------------------------------------


// -------------------------------------------------------------
// common values

float4x4	matWorldViewProj;	
float4x4	matWorld;	
float4		vecLightDir;
float4		vecLightColor;
float4		vecShadowColor;

texture		baseTexture;

sampler sampTexture = sampler_state
{ 
    Texture	= (baseTexture);
    MipFilter	= LINEAR; 
    MinFilter	= LINEAR;
    MagFilter	= LINEAR;
    AddressU	= WRAP;
    AddressV	= WRAP;
};

struct VS_OUTPUT
{
    float4 Pos		: POSITION;
    float4 Diffuse	: TEXCOORD0;
    float2 TexCoord : TEXCOORD1;
    float3 ClipDist	: TEXCOORD2;
};

struct PS_OUTPUT
{
	float4	RGBAColor	: COLOR0;	
};

// -------------------------------------------------------------


// -------------------------------------------------------------
// reflection

float4		clipPlane;

// -------------------------------------------------------------


// -------------------------------------------------------------
// shadow mapping

float4x4	matLightWorldViewProj;

struct VS_OUTPUTCREATESHADOWMAP
{
	float4	Pos		: POSITION;
	float2	Depth	: TEXCOORD0;
	float2	TexCoord	: TEXCOORD1;
};

struct PS_OUTPUTCREATESHADOWMAP
{
	float4	Depth	: COLOR0;
};

// -------------------------------------------------------------




//-----------------------------------------------------------------------------
// Vertex Shader
// + diffuse lighting
// + calculates clipdistance
//-----------------------------------------------------------------------------
VS_OUTPUT VS(	float4 Pos		: POSITION, 
				float3 Normal	: NORMAL, 
				float2 TexCoord : TEXCOORD0, 
				uniform bool bReflect)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;      
    
    // transform Position
    Out.Pos = mul(Pos, matWorldViewProj);	
    
    // transform Normal
    float3 Norm = normalize(mul(Normal, matWorld));
	
	Out.Diffuse =  saturate(dot(Norm, vecLightDir)) * vecLightColor + vecShadowColor;		
	Out.TexCoord = TexCoord;
	
	if(bReflect)
	{
		float4 clipPos =  mul(Pos, matWorld);	
		Out.ClipDist = dot(clipPos, clipPlane.xyz) - clipPlane.w;
	}
	else
	{
		Out.ClipDist = 0;
	}
    
	return Out;
}

//-----------------------------------------------------------------------------
// Pixel Shader
// + clips
// + textures the model
//-----------------------------------------------------------------------------
PS_OUTPUT PS( VS_OUTPUT In,
			  uniform bool bReflect )
{
	PS_OUTPUT Out;
	
	if( bReflect )
	{
		clip(In.ClipDist);
	}
	
	//Out.RGBAColor = tex2D(sampTexture, In.TexCoord) * In.Diffuse;
	//Out.RGBAColor.w = 1.0f;
	
	float4 color = tex2D(sampTexture, In.TexCoord);
	
	clip(color.w - 0.001f);
	
	Out.RGBAColor = color * In.Diffuse;
	Out.RGBAColor.w = color.w;
	
	return Out;
}


//-----------------------------------------------------------------------------
// Vertex Shader
// + prepares for rendering to a depth map
//-----------------------------------------------------------------------------
VS_OUTPUTCREATESHADOWMAP VSCreateShadowMap(	float4 Pos		: POSITION,
											float3 Normal	: NORMAL,
											float2 TexCoord	: TEXCOORD0)
{
    VS_OUTPUTCREATESHADOWMAP Out = (VS_OUTPUTCREATESHADOWMAP)0; 

	// transform light source
	float4 Position = mul( Pos, matLightWorldViewProj);	
	
    // output position
    Out.Pos = Position;
    Out.TexCoord = TexCoord;
    
    // depth value
	// Store z and w in our spare texcoord
	Out.Depth.xy = Out.Pos.zw;

    return Out;
}

//-----------------------------------------------------------------------------
// Pixel Shader
// + renders to a depth map
//-----------------------------------------------------------------------------
PS_OUTPUTCREATESHADOWMAP PSCreateShadowMap(VS_OUTPUTCREATESHADOWMAP In)
{   
	PS_OUTPUTCREATESHADOWMAP Out;
	
//	float4 color = tex2D(sampTexture, In.TexCoord);
    	
//	clip(color.w - 0.001f);
    
    // Depth is z / w
    Out.Depth = In.Depth.x / In.Depth.y;
	
	return Out;
}


// -------------------------------------------------------------
// Name technique | pass | how to compile 
// -------------------------------------------------------------
technique Render
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS(false);
        PixelShader  = compile ps_2_0 PS(false);
    }
}

technique RenderReflection
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS(true);
        PixelShader  = compile ps_2_0 PS(true);
    }
}

technique CreateShadowMap
{
	pass P0
	{
		VertexShader = compile vs_1_1 VSCreateShadowMap();
		PixelShader = compile ps_2_0 PSCreateShadowMap();
	}
}
