// -------------------------------------------------------------
// Terrain Shader
// + dynamic texturing
// + reflection rendering
// + render to shadow map
// -------------------------------------------------------------


// -------------------------------------------------------------
// common values

float4x4	matWorldViewProj;
float4x4	matInvWorld;
float4		lightDir;
float4		diffuseColor;
float4		shadowColor;

struct VS_OUTPUT
{
	float4	Position	: POSITION;
	float4	Diffuse		: COLOR0;
	float2	TexCoord	: TEXCOORD0;
	float3	ClipDist	: TEXCOORD1;
	float	Height		: TEXCOORD3;
	float	Angle		: TEXCOORD4;
};

struct PS_OUTPUT
{
	float4	RGBAColor	: COLOR0;	
};
// -------------------------------------------------------------


// -------------------------------------------------------------
// reflection

float4		clipPlane;
// -------------------------------------------------------------


// -------------------------------------------------------------
// dynamic texturing

const float3	upVec = float3(0.0f, 1.0f, 0.0f);

float		heightScale;

texture		heightTexture1;
texture		heightTexture2;
texture		steepTexture;

float		trans1Factor;
float		trans1End;

float		angleFactor;
float		steep1Factor;
float		steep2Factor;
float		steep1Begin;
float		steep2End;
float		angleBegin;

sampler sampTexture1 = sampler_state
{ 
    Texture		= (heightTexture1);
    MipFilter	= LINEAR; 
    MinFilter	= LINEAR;
    MagFilter	= LINEAR;
    AddressU	= WRAP;
    AddressV	= WRAP;
};

sampler sampTexture2 = sampler_state
{ 
    Texture		= (heightTexture2);
    MipFilter	= LINEAR; 
    MinFilter	= LINEAR;
    MagFilter	= LINEAR;
    AddressU	= WRAP;
    AddressV	= WRAP;
};

sampler sampSteepTexture = sampler_state
{ 
    Texture		= (steepTexture);
    MipFilter	= LINEAR; 
    MinFilter	= LINEAR;
    MagFilter	= LINEAR;
    AddressU	= WRAP;
    AddressV	= WRAP;
};
// -------------------------------------------------------------


// -------------------------------------------------------------
// shadow mapping

float4x4	matLightWorldViewProj;
float		farPlane;
float		nearPlane;

struct VS_OUTPUTCREATESHADOWMAP
{
	float4	Pos		: POSITION;
	float2	Depth	: TEXCOORD0;
};

struct PS_OUTPUTCREATESHADOWMAP
{
	float4	Depth	: COLOR0;
};
// -------------------------------------------------------------



//-----------------------------------------------------------------------------
// Vertex Shader
// + diffuse lighting
// + calculates the steepness
// + calculates clipdistance
//-----------------------------------------------------------------------------
VS_OUTPUT VS	(	float4			Pos			: POSITION, 
					float3			Normal		: NORMAL, 
					float2			TexCoord	: TEXCOORD0,
					uniform bool	bReflect
				)
{
	VS_OUTPUT Output = (VS_OUTPUT) 0; 
	
	Output.Position	= mul(Pos, matWorldViewProj);
	Output.TexCoord	= TexCoord;
	Output.Height	= Pos.y/heightScale;
	   
	float3 Norm		= normalize(mul(matInvWorld, Normal));
	Output.Diffuse	= saturate(dot(Norm, lightDir)*1.0) * diffuseColor + shadowColor;
	
	// Winkel ausrechnen (1.0f = 90, 0.0f = 0)
//	float3 hlp = float3(0,1,0);
	Output.Angle	= 1.0f - abs(dot(Normal, upVec));
	
	if(bReflect)
	{
		Output.ClipDist = dot(Pos, clipPlane.xyz) - clipPlane.w;
	}
	else
	{
		Output.ClipDist = 0;
	}
    
	return Output;
}

//-----------------------------------------------------------------------------
// Pixel Shader
// + clips
// + calculates transparency of the terrain
// + textures the terrain depending on steepness and height
//-----------------------------------------------------------------------------
PS_OUTPUT PS	(	VS_OUTPUT		In,
					uniform bool	bReflect
				)
{
	PS_OUTPUT	Output;
	
	float waterDepth;
	
	if( bReflect )
	{
		clip(In.ClipDist);
		waterDepth = 0;
	}
	else
	{	
		waterDepth = (clipPlane.w - In.Height*heightScale)/clipPlane.w;
	}
	
	float	tex1Factor	= saturate((trans1End - In.Height)*trans1Factor);		
	float4	groundColor	= lerp(tex2D(sampTexture2, In.TexCoord), tex2D(sampTexture1, In.TexCoord), tex1Factor);
	
	//float	steepFactor_up	= (In.Height - steep1Begin)*steep1Factor;
	//float	steepFactor_dn	= (steep2End - In.Height)*steep2Factor;
	//float	steepFactor	= saturate(min(steepFactor_up, steepFactor_dn));
	
	float	steepAngleFactor = saturate((In.Angle - angleBegin)*angleFactor);
	
	//float	a = steepFactor * steepAngleFactor;
	//float4 steepColor = a * tex2D(sampSteepTexture, In.TexCoord);	
	
	float4 steepColor = steepAngleFactor * tex2D(sampSteepTexture, In.TexCoord);	
	
	//Output.RGBAColor	= In.Diffuse * lerp(groundColor, steepColor, a);
	Output.RGBAColor	= In.Diffuse * lerp(groundColor, steepColor, steepAngleFactor);
	Output.RGBAColor.w	= 1.0f - waterDepth;
	
	return Output;
}


//-----------------------------------------------------------------------------
// Vertex Shader
// + diffuse lighting
//-----------------------------------------------------------------------------
//VS_OUTPUT VSSimple	(	float4			Pos			: POSITION, 
//						float3			Normal		: NORMAL, 
//						float2			TexCoord	: TEXCOORD0
//					)
//{
//	VS_OUTPUT Output = (VS_OUTPUT) 0; 
//
//	Output.Position	= mul(Pos, matWorldViewProj);
//	Output.TexCoord	= TexCoord;
//	Output.Height	= Pos.y/heightScale;
//	   
//	float3 Norm		= normalize(mul(matInvWorld, Normal));
//	Output.Diffuse	= saturate(dot(Norm, lightDir)) * diffuseColor + shadowColor;
//    
//	return Output;
//}

//-----------------------------------------------------------------------------
// Pixel Shader
// + colors
//-----------------------------------------------------------------------------
//PS_OUTPUT PSSimple	(	VS_OUTPUT		In 	)
//{
//	PS_OUTPUT	Output;
//	
//	Output.RGBAColor	= In.Diffuse;
//	Output.RGBAColor.w	= 1.0f;
//	
//	return Output;
//}


//-----------------------------------------------------------------------------
// Vertex Shader
// + prepares for rendering to a depth map
//-----------------------------------------------------------------------------
VS_OUTPUTCREATESHADOWMAP VSCreateShadowMap(	float4 Pos		: POSITION,
											float3 Normal	: NORMAL,
											float2 TexCoord	: TEXCOORD0)
{
    VS_OUTPUTCREATESHADOWMAP Out = (VS_OUTPUTCREATESHADOWMAP)0; 

	// transform light source
	float4 Position = mul( Pos, matLightWorldViewProj);	
	
    // output position
    Out.Pos = Position;
    
    // depth value
    Out.Depth = Out.Pos.zw;

    return Out;
}

//-----------------------------------------------------------------------------
// Pixel Shader
// + renders to a depth map
//-----------------------------------------------------------------------------
PS_OUTPUTCREATESHADOWMAP PSCreateShadowMap(VS_OUTPUTCREATESHADOWMAP In)
{   
	PS_OUTPUTCREATESHADOWMAP Out;
	
	// Depth is z / w
	Out.Depth = In.Depth.x / In.Depth.y;
	
	return Out;
}

// -------------------------------------------------------------
// Name technique | pass | how to compile 
// -------------------------------------------------------------
technique Render
{
    pass P0
    {        
        VertexShader = compile vs_2_0 VS(false);
        PixelShader  = compile ps_2_0 PS(false);
    }
}

technique RenderReflection
{
    pass P0
    {        
        VertexShader = compile vs_2_0 VS(true);
        PixelShader  = compile ps_2_0 PS(true);
    }
}

//technique RenderSimple
//{
//    pass P0
//    {        
//        VertexShader = compile vs_1_1 VSSimple();
//        PixelShader  = compile ps_1_1 PSSimple();
//    }
//}

technique CreateShadowMap
{
	pass P0
	{
		VertexShader = compile vs_1_1 VSCreateShadowMap();
		PixelShader = compile ps_2_0 PSCreateShadowMap();
	}
}