// -------------------------------------------------------------
// variables that are provided by the application
// -------------------------------------------------------------
float4x4	matWorldViewProj;	
float4x4	matWorld;
float4x4	matProjTex;
float4		lightDir;
float4		eyePt;
float4		diffuseColor;
float4		lightColor;
float		time;
float		waveScale;
float		distortionBigWaves;
float		distortionSmallWaves;
float4		windDirection;
float		specularExponent;
float		waterHeight;
float		specularScale;

Texture		waterTexture;
Texture		waveTexture;
Texture		cubeMap;

sampler sampWater = sampler_state 
{ 
	texture = <waterTexture>; 
    minfilter = LINEAR; 
    mipfilter = LINEAR; 
    magfilter = LINEAR;
    addressu = CLAMP;
    addressv = CLAMP;
    addressw = CLAMP;
};

sampler sampWave = sampler_state 
{ 
	texture = <waveTexture>; 
    minfilter = LINEAR; 
    mipfilter = LINEAR; 
    magfilter = LINEAR;
    addressu = WRAP;
    addressv = WRAP;
    addressw = WRAP;
};

samplerCUBE sampCube = sampler_state
{
   Texture = <cubeMap>;
   MinFilter = LINEAR;
   MagFilter = LINEAR;
   AddressU  = CLAMP;
   AddressV  = CLAMP;
   AddressW  = CLAMP;   
};

// -------------------------------------------------------------
// vertex shader output channels
// -------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float3 Norm : TEXCOORD0;
    float3 Incident : TEXCOORD1;
    float4 TexCoords : TEXCOORD2;
    float2 WaveCoords : TEXCOORD3;
    float3 Half : TEXCOORD4;
};

//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
	float4	RGBAColor	: COLOR0;	
};

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------

VS_OUTPUT VS(float4 Pos : POSITION, float3 Normal : NORMAL)
{
	VS_OUTPUT Out = (VS_OUTPUT) 0;
	
	Out.Pos = mul(Pos, matWorldViewProj);								// transform Position
	Out.Norm = normalize(mul(Normal, matWorld));
	
	float4 PosWorld = mul(Pos, matWorld);
	float3 View = normalize(eyePt - PosWorld);
	float3 LightDir = float3(lightDir.x, lightDir.y, lightDir.z);
	
	Out.Half = normalize(LightDir + View);	
	Out.Incident = normalize(PosWorld - eyePt);
	   
	float4	dPos;
	dPos.xz			= Pos.xz + distortionBigWaves * normalize(Normal).xz;
	dPos.y			= waterHeight;
	dPos.w			= 1.0;
	Out.TexCoords	= dPos;
	
	Out.WaveCoords.x = Pos.x/waveScale + windDirection.x * time/waveScale;
	Out.WaveCoords.y = Pos.z/waveScale + windDirection.y * time/waveScale;
	    
	return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
PS_OUTPUT PS(VS_OUTPUT	In, uniform bool isSunVisible) : COLOR0
{ 
	PS_OUTPUT Output;
	

	float3	Helper		= {0, 2, 0}; // zur Abschwchung
	
	float3	waveColor	= tex2D(sampWave, In.WaveCoords);
	float3	waveNormals	= normalize((2.0f * waveColor) - 1.0f + Helper);// + Helper;
//	float3	NormalSpec	= normalize(waveNormals + In.Norm);
	 
	float4	specColor;
    
	if(isSunVisible)
	{
		// specular component
		In.Half = normalize(In.Half);
		float3	NormalSpec	= normalize(waveNormals + In.Norm);
		
		float	Spec		= saturate(dot(NormalSpec, In.Half));
//		float	Diff		= saturate(dot(NormalSpec, In.Light));
		
//		specColor	= pow(Spec,specularExponent)*lightColor*Diff;
		specColor	= pow(Spec,specularExponent)*specularScale*lightColor;
	}
	else
	{
		specColor	= (0.0f, 0.0f, 0.0f, 0.0f);
	}
    
	float3	NormalSky	= normalize(waveNormals*0.5f+In.Norm);
    
    // v = i - 2 * dot(i, n) * n ... v = reflection, i = incident, n = normal
	float3	Reflect		= reflect(In.Incident, NormalSky);
	float4	skyColor	= texCUBE(sampCube, Reflect);
        
	In.TexCoords.xz			= In.TexCoords.xz + distortionSmallWaves * waveNormals.xz;
	In.TexCoords			= mul(In.TexCoords, matProjTex);
	float4	reflectColor	= tex2Dproj(sampWater, In.TexCoords);
    
	float4	waterColor				= lerp(lerp(diffuseColor, skyColor, 0.8f), lerp(lerp(diffuseColor, skyColor, 0.8f), reflectColor, 0.5f), reflectColor.w);
	
	Output.RGBAColor	= waterColor + specColor*0.7f;
	Output.RGBAColor.w	= 0.7f;												

	return Output;
}

// -------------------------------------------------------------
// Name technique | pass | how to compile 
// -------------------------------------------------------------
technique RenderWithSunReflection
{
    pass P0
    {        
        // compiler directives
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS(true);
    }
}

technique RenderWithoutSunReflection
{
    pass P0
    {       
        // compiler directives
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS(false);
    }
}
